@@ -30,7 +30,7 @@
 
 #include "glib.h"
 #include "gprintfint.h"
-#include "gthreadinit.h"
+#include "gthreadprivate.h"
 
 #ifdef G_PLATFORM_WIN32
 #define STRICT
@@ -52,11 +52,7 @@
 GQuark 
 g_convert_error_quark (void)
 {
-  static GQuark quark;
-  if (!quark)
-    quark = g_quark_from_static_string ("g_convert_error");
-
-  return quark;
+  return g_quark_from_static_string ("g_convert_error");
 }
 
 static gboolean
@@ -196,6 +192,8 @@ g_iconv_close (GIConv converter)
 }
 
 
+#ifdef NEED_ICONV_CACHE
+
 #define ICONV_CACHE_SIZE   (16)
 
 struct _iconv_cache_bucket {
@@ -463,6 +461,47 @@ close_converter (GIConv converter)
   return 0;
 }
 
+#else  /* !NEED_ICONV_CACHE */
+
+static GIConv
+open_converter (const gchar *to_codeset,
+		const gchar *from_codeset,
+		GError     **error)
+{
+  GIConv cd;
+
+  cd = g_iconv_open (to_codeset, from_codeset);
+
+  if (cd == (GIConv) -1)
+    {
+      /* Something went wrong.  */
+      if (error)
+	{
+	  if (errno == EINVAL)
+	    g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_NO_CONVERSION,
+			 _("Conversion from character set '%s' to '%s' is not supported"),
+			 from_codeset, to_codeset);
+	  else
+	    g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_FAILED,
+			 _("Could not open converter from '%s' to '%s'"),
+			 from_codeset, to_codeset);
+	}
+    }
+  
+  return cd;
+}
+
+static int
+close_converter (GIConv cd)
+{
+  if (cd == (GIConv) -1)
+    return 0;
+  
+  return g_iconv_close (cd);  
+}
+
+#endif /* NEED_ICONV_CACHE */
+
 /**
  * g_convert_with_iconv:
  * @str:           the string to convert
@@ -1792,7 +1831,8 @@ g_filename_from_uri (const gchar *uri,
  * @error: location to store the error occuring, or %NULL to ignore
  *         errors. Any of the errors in #GConvertError may occur.
  * 
- * Converts an absolute filename to an escaped ASCII-encoded URI.
+ * Converts an absolute filename to an escaped ASCII-encoded URI, with the path
+ * component following Section 3.3. of RFC 2396.
  * 
  * Return value: a newly-allocated string holding the resulting
  *               URI, or %NULL on an error.
@@ -1953,7 +1993,8 @@ make_valid_utf8 (const gchar *name)
 	string = g_string_sized_new (remaining_bytes);
 
       g_string_append_len (string, remainder, valid_bytes);
-      g_string_append_c (string, '?');
+      /* append U+FFFD REPLACEMENT CHARACTER */
+      g_string_append (string, "\357\277\275");
       
       remaining_bytes -= valid_bytes + 1;
       remainder = invalid + 1;
@@ -1963,7 +2004,6 @@ make_valid_utf8 (const gchar *name)
     return g_strdup (name);
   
   g_string_append (string, remainder);
-  g_string_append (string, " (invalid encoding)");
 
   g_assert (g_utf8_validate (string->str, -1, NULL));
   
@@ -1977,7 +2017,13 @@ make_valid_utf8 (const gchar *name)
  * Returns the display basename for the particular filename, guaranteed
  * to be valid UTF-8. The display name might not be identical to the filename,
  * for instance there might be problems converting it to UTF-8, and some files
- * can be translated in the display
+ * can be translated in the display.
+ *
+ * If GLib can not make sense of the encoding of @filename, as a last resort it 
+ * replaces unknown characters with U+FFFD, the Unicode replacement character.
+ * You can search the result for the UTF-8 encoding of this character (which is
+ * "\357\277\275" in octal notation) to find out if @filename was in an invalid
+ * encoding.
  *
  * You must pass the whole absolute pathname to this functions so that
  * translation of well known locations can be done.
@@ -2008,13 +2054,17 @@ g_filename_display_basename (const gchar *filename)
  * g_filename_display_name:
  * @filename: a pathname hopefully in the GLib file name encoding
  * 
- * Converts a filename into a valid UTF-8 string. The 
- * conversion is not necessarily reversible, so you 
- * should keep the original around and use the return
- * value of this function only for display purposes.
- * Unlike g_filename_to_utf8(), the result is guaranteed 
- * to be non-NULL even if the filename actually isn't in the GLib
- * file name encoding.
+ * Converts a filename into a valid UTF-8 string. The conversion is 
+ * not necessarily reversible, so you should keep the original around 
+ * and use the return value of this function only for display purposes.
+ * Unlike g_filename_to_utf8(), the result is guaranteed to be non-%NULL 
+ * even if the filename actually isn't in the GLib file name encoding.
+ *
+ * If GLib can not make sense of the encoding of @filename, as a last resort it 
+ * replaces unknown characters with U+FFFD, the Unicode replacement character.
+ * You can search the result for the UTF-8 encoding of this character (which is
+ * "\357\277\275" in octal notation) to find out if @filename was in an invalid
+ * encoding.
  *
  * If you know the whole pathname of the file you should use
  * g_filename_display_basename(), since that allows location-based
