@@ -152,6 +152,9 @@ static gint                  g_key_file_parse_value_as_integer (GKeyFile
 								GError                **error);
 static gchar                *g_key_file_parse_integer_as_value (GKeyFile               *key_file,
 								gint                    value);
+static gdouble               g_key_file_parse_value_as_double  (GKeyFile               *key_file,
+                                                                const gchar            *value,
+                                                                GError                **error);
 static gboolean              g_key_file_parse_value_as_boolean (GKeyFile               *key_file,
 								const gchar            *value,
 								GError                **error);
@@ -185,12 +188,7 @@ static void                  g_key_file_flush_parse_buffer     (GKeyFile
 GQuark
 g_key_file_error_quark (void)
 {
-  static GQuark error_quark = 0;
-
-  if (error_quark == 0)
-    error_quark = g_quark_from_static_string ("g-key-file-error-quark");
-
-  return error_quark;
+  return g_quark_from_static_string ("g-key-file-error-quark");
 }
 
 static void
@@ -931,7 +929,6 @@ g_key_file_to_data (GKeyFile  *key_file,
 		    GError   **error)
 {
   GString *data_string;
-  gchar *data;
   GList *group_node, *key_file_node;
 
   g_return_val_if_fail (key_file != NULL, NULL);
@@ -946,6 +943,10 @@ g_key_file_to_data (GKeyFile  *key_file,
 
       group = (GKeyFileGroup *) group_node->data;
 
+      /* separate groups by an empty line */
+      if (group_node->next)
+        g_string_append_c (data_string, '\n');
+
       if (group->comment != NULL)
         g_string_append_printf (data_string, "%s\n", group->comment->value);
       if (group->name != NULL)
@@ -969,11 +970,7 @@ g_key_file_to_data (GKeyFile  *key_file,
   if (length)
     *length = data_string->len;
 
-  data = data_string->str;
-
-  g_string_free (data_string, FALSE);
-
-  return data;
+  return g_string_free (data_string, FALSE);
 }
 
 /**
@@ -1107,7 +1104,7 @@ g_key_file_get_groups (GKeyFile *key_file,
    * list) is always the comment group at the top,
    * which we skip
    */
-  groups = (gchar **) g_new0 (gchar **, num_groups);
+  groups = g_new0 (gchar *, num_groups);
 
   group_node = g_list_last (key_file->groups);
   
@@ -1466,7 +1463,7 @@ g_key_file_set_string_list (GKeyFile            *key_file,
   g_return_if_fail (list != NULL);
 
   value_list = g_string_sized_new (length * 128);
-  for (i = 0; list[i] != NULL && i < length; i++)
+  for (i = 0; i < length && list[i] != NULL; i++)
     {
       gchar *value;
 
@@ -1714,7 +1711,7 @@ g_key_file_set_locale_string_list (GKeyFile            *key_file,
   g_return_if_fail (length != 0);
 
   value_list = g_string_sized_new (length * 128);
-  for (i = 0; list[i] != NULL && i < length; i++)
+  for (i = 0; i < length && list[i] != NULL; i++)
     {
       gchar *value;
       
@@ -2158,6 +2155,218 @@ g_key_file_set_integer_list (GKeyFile     *key_file,
   g_string_free (values, TRUE);
 }
 
+/**
+ * g_key_file_get_double:
+ * @key_file: a #GKeyFile
+ * @group_name: a group name
+ * @key: a key
+ * @error: return location for a #GError
+ *
+ * Returns the value associated with @key under @group_name as an
+ * integer. If @group_name is %NULL, the start_group is used.
+ *
+ * If @key cannot be found then the return value is undefined and
+ * @error is set to #G_KEY_FILE_ERROR_KEY_NOT_FOUND. Likewise, if
+ * the value associated with @key cannot be interpreted as a double
+ * then the return value is also undefined and @error is set to
+ * #G_KEY_FILE_ERROR_INVALID_VALUE.
+ *
+ * Return value: the value associated with the key as a double.
+ *
+ * Since: 2.12
+ **/
+gdouble
+g_key_file_get_double  (GKeyFile     *key_file,
+                        const gchar  *group_name,
+                        const gchar  *key,
+                        GError      **error)
+{
+  GError *key_file_error;
+  gchar *value;
+  gdouble double_value;
+
+  g_return_val_if_fail (key_file != NULL, -1);
+  g_return_val_if_fail (group_name != NULL, -1);
+  g_return_val_if_fail (key != NULL, -1);
+
+  key_file_error = NULL;
+
+  value = g_key_file_get_value (key_file, group_name, key, &key_file_error);
+
+  if (key_file_error)
+    {
+      g_propagate_error (error, key_file_error);
+      return 0;
+    }
+
+  double_value = g_key_file_parse_value_as_double (key_file, value,
+                                                  &key_file_error);
+  g_free (value);
+
+  if (key_file_error)
+    {
+      if (g_error_matches (key_file_error,
+                           G_KEY_FILE_ERROR,
+                           G_KEY_FILE_ERROR_INVALID_VALUE))
+        {
+          g_set_error (error, G_KEY_FILE_ERROR,
+                       G_KEY_FILE_ERROR_INVALID_VALUE,
+                       _("Key file contains key '%s' in group '%s' "
+                         "which has value that cannot be interpreted."), key,
+                       group_name);
+          g_error_free (key_file_error);
+        }
+      else
+        g_propagate_error (error, key_file_error);
+    }
+
+  return double_value;
+}
+
+/**
+ * g_key_file_set_double:
+ * @key_file: a #GKeyFile
+ * @group_name: a group name
+ * @key: a key
+ * @value: an double value
+ *
+ * Associates a new double value with @key under @group_name.
+ * If @key cannot be found then it is created. If @group_name
+ * is %NULL, the start group is used.
+ *
+ * Since: 2.12
+ **/
+void
+g_key_file_set_double  (GKeyFile    *key_file,
+                        const gchar *group_name,
+                        const gchar *key,
+                        gdouble      value)
+{
+  gchar result[G_ASCII_DTOSTR_BUF_SIZE];
+
+  g_return_if_fail (key_file != NULL);
+  g_return_if_fail (group_name != NULL);
+  g_return_if_fail (key != NULL);
+
+  g_ascii_dtostr ( result, sizeof (result), value );
+  g_key_file_set_value (key_file, group_name, key, result);
+}
+
+/**
+ * g_key_file_get_double_list:
+ * @key_file: a #GKeyFile
+ * @group_name: a group name
+ * @key: a key
+ * @length: the number of doubles returned
+ * @error: return location for a #GError
+ *
+ * Returns the values associated with @key under @group_name as
+ * doubles. If @group_name is %NULL, the start group is used.
+ *
+ * If @key cannot be found then the return value is undefined and
+ * @error is set to #G_KEY_FILE_ERROR_KEY_NOT_FOUND. Likewise, if
+ * the values associated with @key cannot be interpreted as doubles
+ * then the return value is also undefined and @error is set to
+ * #G_KEY_FILE_ERROR_INVALID_VALUE.
+ *
+ * Return value: the values associated with the key as a double
+ *
+ * Since: 2.12
+ **/
+gdouble *
+g_key_file_get_double_list  (GKeyFile     *key_file,
+                             const gchar  *group_name,
+                             const gchar  *key,
+                             gsize        *length,
+                             GError      **error)
+{
+  GError *key_file_error = NULL;
+  gchar **values;
+  gdouble *double_values;
+  gsize i, num_doubles;
+
+  g_return_val_if_fail (key_file != NULL, NULL);
+  g_return_val_if_fail (group_name != NULL, NULL);
+  g_return_val_if_fail (key != NULL, NULL);
+
+  values = g_key_file_get_string_list (key_file, group_name, key,
+                                       &num_doubles, &key_file_error);
+
+  if (key_file_error)
+    g_propagate_error (error, key_file_error);
+
+  if (!values)
+    return NULL;
+
+  double_values = g_new0 (gdouble, num_doubles);
+
+  for (i = 0; i < num_doubles; i++)
+    {
+      double_values[i] = g_key_file_parse_value_as_double (key_file,
+							   values[i],
+							   &key_file_error);
+
+      if (key_file_error)
+        {
+          g_propagate_error (error, key_file_error);
+          g_strfreev (values);
+          g_free (double_values);
+
+          return NULL;
+        }
+    }
+  g_strfreev (values);
+
+  if (length)
+    *length = num_doubles;
+
+  return double_values;
+}
+
+/**
+ * g_key_file_set_double_list:
+ * @key_file: a #GKeyFile
+ * @group_name: a group name
+ * @key: a key
+ * @list: an array of double values
+ * @length: number of double values in @list
+ *
+ * Associates a list of double values with @key under
+ * @group_name.  If @key cannot be found then it is created.
+ * If @group_name is %NULL the start group is used.
+ *
+ * Since: 2.21
+ **/
+void
+g_key_file_set_double_list (GKeyFile     *key_file,
+			    const gchar  *group_name,
+			    const gchar  *key,
+			    gdouble       list[],
+			    gsize         length)
+{
+  GString *values;
+  gsize i;
+
+  g_return_if_fail (key_file != NULL);
+  g_return_if_fail (group_name != NULL);
+  g_return_if_fail (key != NULL);
+  g_return_if_fail (list != NULL);
+
+  values = g_string_sized_new (length * 16);
+  for (i = 0; i < length; i++)
+    {
+      gchar result[G_ASCII_DTOSTR_BUF_SIZE];
+
+      g_ascii_dtostr( result, sizeof (result), list[i] );
+
+      g_string_append (values, result);
+      g_string_append_c (values, ';');
+    }
+
+  g_key_file_set_value (key_file, group_name, key, values->str);
+  g_string_free (values, TRUE);
+}
+
 static void
 g_key_file_set_key_comment (GKeyFile             *key_file,
                             const gchar          *group_name,
@@ -2489,8 +2698,7 @@ g_key_file_get_group_comment (GKeyFile             *key_file,
   GKeyFileGroup *group;
   
   group_node = g_key_file_lookup_group_node (key_file, group_name);
-  group = (GKeyFileGroup *)group_node->data;
-  if (!group)
+  if (!group_node)
     {
       g_set_error (error, G_KEY_FILE_ERROR,
                    G_KEY_FILE_ERROR_GROUP_NOT_FOUND,
@@ -2500,6 +2708,7 @@ g_key_file_get_group_comment (GKeyFile             *key_file,
       return NULL;
     }
 
+  group = (GKeyFileGroup *)group_node->data;
   if (group->comment)
     return g_strdup (group->comment->value);
   
@@ -2665,7 +2874,9 @@ g_key_file_add_group (GKeyFile    *key_file,
 
   g_return_if_fail (key_file != NULL);
   g_return_if_fail (group_name != NULL);
-  g_return_if_fail (g_key_file_lookup_group_node (key_file, group_name) == NULL);
+
+  if (g_key_file_lookup_group_node (key_file, group_name) != NULL)
+    return;
 
   group = g_new0 (GKeyFileGroup, 1);
   group->name = g_strdup (group_name);
@@ -2907,15 +3118,12 @@ g_key_file_lookup_group_node (GKeyFile    *key_file,
   GKeyFileGroup *group;
   GList *tmp;
 
-  group = NULL;
   for (tmp = key_file->groups; tmp != NULL; tmp = tmp->next)
     {
       group = (GKeyFileGroup *) tmp->data;
 
       if (group && group->name && strcmp (group->name, group_name) == 0)
         break;
-
-      group = NULL;
     }
 
   return tmp;
@@ -3252,6 +3460,24 @@ g_key_file_parse_integer_as_value (GKeyFile *key_file,
   return g_strdup_printf ("%d", value);
 }
 
+static gdouble
+g_key_file_parse_value_as_double  (GKeyFile     *key_file,
+                                   const gchar  *value,
+                                   GError      **error)
+{
+  gchar *end_of_valid_d;
+  gdouble double_value = 0;
+
+  double_value = g_ascii_strtod (value, &end_of_valid_d);
+
+  if (*end_of_valid_d != '\0' || end_of_valid_d == value)
+    g_set_error (error, G_KEY_FILE_ERROR,
+                 G_KEY_FILE_ERROR_INVALID_VALUE,
+                 _("Value '%s' cannot be interpreted as a float number."), value);
+
+  return double_value;
+}
+
 static gboolean
 g_key_file_parse_value_as_boolean (GKeyFile     *key_file,
 				   const gchar  *value,
@@ -3287,7 +3513,7 @@ g_key_file_parse_value_as_comment (GKeyFile    *key_file,
                                    const gchar *value)
 {
   GString *string;
-  gchar **lines, *comment;
+  gchar **lines;
   gsize i;
 
   string = g_string_sized_new (512);
@@ -3303,11 +3529,7 @@ g_key_file_parse_value_as_comment (GKeyFile    *key_file,
     }
   g_strfreev (lines);
 
-  comment = string->str;
-
-  g_string_free (string, FALSE);
-
-  return comment;
+  return g_string_free (string, FALSE);
 }
 
 static gchar *
@@ -3315,7 +3537,7 @@ g_key_file_parse_comment_as_value (GKeyFile      *key_file,
                                    const gchar   *comment)
 {
   GString *string;
-  gchar **lines, *value;
+  gchar **lines;
   gsize i;
 
   string = g_string_sized_new (512);
@@ -3327,11 +3549,7 @@ g_key_file_parse_comment_as_value (GKeyFile      *key_file,
                             lines[i + 1] == NULL? "" : "\n");
   g_strfreev (lines);
 
-  value = string->str;
-
-  g_string_free (string, FALSE);
-
-  return value;
+  return g_string_free (string, FALSE);
 }
 
 #define __G_KEY_FILE_C__
