@@ -20,10 +20,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
+
+#include <string.h>
 
 #include "glib.h"
-#include <string.h>
+#include "galias.h"
 
 #ifdef _
 #warning "FIXME remove gettext hack"
@@ -34,10 +36,7 @@
 GQuark
 g_shell_error_quark (void)
 {
-  static GQuark quark = 0;
-  if (quark == 0)
-    quark = g_quark_from_static_string ("g-shell-error-quark");
-  return quark;
+  return g_quark_from_static_string ("g-shell-error-quark");
 }
 
 /* Single quotes preserve the literal string exactly. escape
@@ -257,7 +256,7 @@ g_shell_unquote (const gchar *quoted_string,
 
   start = unquoted;
   end = unquoted;
-  retval = g_string_new ("");
+  retval = g_string_new (NULL);
 
   /* The loop allows cases such as
    * "foo"blah blah'bar'woo foo"baz"la la la\'\''foo'
@@ -305,6 +304,7 @@ g_shell_unquote (const gchar *quoted_string,
         }
     }
 
+  g_free (unquoted);
   return g_string_free (retval, FALSE);
   
  error:
@@ -388,7 +388,7 @@ static inline void
 ensure_token (GString **token)
 {
   if (*token == NULL)
-    *token = g_string_new ("");
+    *token = g_string_new (NULL);
 }
 
 static void
@@ -411,10 +411,12 @@ tokenize_command_line (const gchar *command_line,
   const gchar *p;
   GString *current_token = NULL;
   GSList *retval = NULL;
-  
-  current_quote = '\0';
-  p = command_line;
+  gboolean quoted;;
 
+  current_quote = '\0';
+  quoted = FALSE;
+  p = command_line;
+ 
   while (*p)
     {
       if (current_quote == '\\')
@@ -450,7 +452,7 @@ tokenize_command_line (const gchar *command_line,
         {
           if (*p == current_quote &&
               /* check that it isn't an escaped double quote */
-              !(current_quote == '"' && p != command_line && *(p - 1) == '\\'))
+              !(current_quote == '"' && quoted))
             {
               /* close the quote */
               current_quote = '\0';
@@ -514,6 +516,14 @@ tokenize_command_line (const gchar *command_line,
             }
         }
 
+      /* We need to count consecutive backslashes mod 2, 
+       * to detect escaped doublequotes.
+       */
+      if (*p != '\\')
+	quoted = FALSE;
+      else
+	quoted = !quoted;
+
       ++p;
     }
 
@@ -525,7 +535,7 @@ tokenize_command_line (const gchar *command_line,
         g_set_error (error,
                      G_SHELL_ERROR,
                      G_SHELL_ERROR_BAD_QUOTING,
-                     _("Text ended just after a '\' character."
+                     _("Text ended just after a '\\' character."
                        " (The text was '%s')"),
                      command_line);
       else
@@ -658,3 +668,6 @@ g_shell_parse_argv (const gchar *command_line,
   
   return FALSE;
 }
+
+#define __G_SHELL_C__
+#include "galiasdef.c"
